;
; _Fastest_ ZCP Decoder v1.0 - by CrASH_Man (Hideaki Omuro)   made: 09/30/97
;
; Documentation (03/04/98):
;
;   I have used GARF.ZCP included with ZCP.ZIP that Wouter Demuynck released.
;
;   This is simple to use: Point HL to place to decompress (GRAPH_MEM)
;                          Point DE to compressed pic      (ZCP stream)
;
;   Important: The calculator's interrupts MUST be disabled with DI.
;              This routine probably destroys all registers :)
;
;   Please note that this routine is rather large (95 bytes) and is not at
;   all space-optimized (all are time optimizations).  Thanks.
;
;   If anyone can prove me wrong (that this is the fastest) please send me
;   a mail disproving it.  The original (the one I made for TI-85 first)
;   obviously beat Andreas Ess's Fast ZCP decoder and Pascal Bouron's ZCP
;   2.0.  If anyone can beat this one, please mail me.  Thanks :)
;
; Please send all bug reports, questions, etc. to:
;
;                                        [CrASH_Man]
;                                          crashman@technologist.com
;                                        [CrASH Landing]
;                                          http://crash82.home.ml.org
;
; Original ZCP Idea by Stephane Jantzen 12/16/95
;   Stephane.Jantzen@scinfo.u-nancy.fr (if it still exists)
;

;
; ShowZCP
;
; HL points to place to decompress ZCP to.
; DE points to ZCP stream
; DE -> Picture Stream, HL -> Picture
; DI must be set to work, as this routine uses EX AF, AF'
ShowZCP:
  push hl
  push de
  call _grbufclr	; clear graph screen
  pop de
  pop hl
  DI                    ; This line is REQUIRED
  LD  C, %10000000      ; C contains graphics mask
_ZCP_DecodeLoop:
  EX  AF, AF'           ; A contains junk data
_ZCP_DecodeLoop2:
  LD  A, (DE)           ; A = next byte of compressed data
  OR  A                 ; If zero, end
  JR  Z, _ZCP_EndDecode

  LD  B, A              ; B = byte
  INC DE                ; Next time, read next byte from data stream
  ADD A, A              ; Is MSB set?
  JR  C, _ZCP_Copy      ; Yes, copy...
  ADD A, A              ; Is MSB set now?
  JR  NC, _ZCP_WhiteStream ; No, Do a stream of white pixels

_ZCP_BlackStream:
  EX  AF, AF'           ; A contains screen data
  RES 6, B              ; B < 63

_ZCP_BSLoop:
  OR  C                 ; Set Bit
  RRC C                 ; Shift the Mask
  JR  C, _ZCP_BSNextByte
_ZCP_BSSameByte:
  DJNZ _ZCP_BSLoop
  JR  _ZCP_DecodeLoop
_ZCP_BSNextByte:
  LD  (HL), A           ; Save Screen data
  INC HL
  XOR A                 ; Next one is blank
  DJNZ _ZCP_BSLoop
  JR  _ZCP_DecodeLoop

_ZCP_WhiteStream:
  EX  AF, AF'           ; A contains screen data
  LD  (HL), A           ; write data
_ZCP_WSLoop:
  RRC C
  JR  C, _ZCP_WSNextByte
_ZCP_WSSameByte:
  DJNZ _ZCP_WSLoop
  LD  A, (HL)           ; read next - useful when stream <= 7
  JR  _ZCP_DecodeLoop
_ZCP_WSNextByte:
  INC HL
  DJNZ _ZCP_WSLoop
  LD  A, (HL)           ; read next - useful when stream <= 7
  JR  _ZCP_DecodeLoop

_ZCP_Copy:
  LD  B, $07
_ZCP_CLoop:
  ADD A, A
  JR  NC, _ZCP_CNoCopy
_ZCP_CCopy:
  EX  AF, AF'
  OR  C
  EX  AF, AF'
_ZCP_CNoCopy:
  RRC C
  JR  C, _ZCP_CNextByte
_ZCP_CSameByte:
  DJNZ _ZCP_CLoop
  JR  _ZCP_DecodeLoop2
_ZCP_CNextByte:
  EX  AF, AF'
  LD  (HL), A
  INC HL
  XOR A
  EX  AF, AF'
  DJNZ _ZCP_CLoop
  JR  _ZCP_DecodeLoop2
_ZCP_EndDecode:
  EX  AF, AF'
  LD  (HL), A
  EI                    ; May remove if unneeded
  RET
